# -*- coding: utf-8 -*-
#
# Copyright (C) 2021 CERN.
# Copyright (C) 2021 TU Wien.
#
# Invenio-Requests is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Version information for Invenio-Requests.

This file is imported by ``invenio_requests.__init__``,
and parsed by ``setup.py``.
"""

__version__ = "0.2.4"
